#include <string.h>
#include "license_action.h"
#include "license_logging.h"
#include "xalloc.h"

bool op5lic_license_action_subsetof(const char *sub, const char *super) {
	const char *a_p = sub;
	const char *b_p = super;
	if (!sub || !super || *a_p == '\0' || *b_p == '\0') {
		return false;
	}
	log_debug("testing inclusion of '%s' in '%s'", sub, super);
	while (1) {
		if (*b_p == ':') {
			/* we've reached the midpoint, we must have reached the end of the corresponding token
			 * in the subset, or we're dead - but only if we have actually moved anywhere in B. Otherwise
			 * we're comparing to the root, which is a superset to all
			 * */
			if (b_p != super && *a_p != ':' && *a_p != '.') {

				log_debug("resource in %s is not a subset of resource in %s (difference at position %d)", sub, super, b_p - super);
				return false;

			}

			if (*(b_p+1) == '\0'){
				/* B has method scope root */
				break;
			}

			while (*a_p != ':') /* fast foward! let A catch up to B */
				a_p++;

		} else if (*b_p == '\0') {
			/* we've reached the end of the superset, we must have reached the
			 * end of the corresponding token in the subset, or we're dead */
			if (*a_p != '\0' && *a_p != '.') {
				log_debug("resource in %s is not a subset of resource in %s (difference at position %d)", sub, super, b_p - super);
				return false;
			}
			break;
		}

		if (*a_p != *b_p){
			log_debug("resource in %s is not a subset of resource in %s (difference at position %d)", sub, super, b_p - super);
			return false;
		}
		a_p++;
		b_p++;
	}
	log_debug("%s is a subset of %s", sub, super);
	return true;
}
