#ifndef OP5_METRIC_LIST_H
#define OP5_METRIC_LIST_H
struct metric_list_t;
struct metric_node_t;

/*
 * Creates a new metric list and returns a handle to it.
 */
struct metric_list_t * metric_list_new(void);

/*
 * Copies an existing metric list and returns a handle to the copy.
 */
struct metric_list_t * metric_list_copy(const struct metric_list_t *list);

/*
 * Frees the supplied list and all of nodes and associated values within it.
 */
void metric_list_destroy(struct metric_list_t *list);
/*
 * Returns a handle to the first element in the supplied list, if any, or NULL if none.
 */
struct metric_node_t * metric_list_head(struct metric_list_t *);
/*
 * Adds a metric to the supplied list using the specified values for `value', `min' and `max'
 */
struct metric_node_t * metric_list_add(struct metric_list_t *list, const char *id, int value, int min, int max);
/*
 * Returns a handle to the subsequent node of `node', if any, or NULL if none.
 */
struct metric_node_t * metric_node_next(struct metric_node_t *node);

/*
 * Returns the ID for the metric kept at `node'
 */
const char * metric_node_id(struct metric_node_t *node);
/*
 * Returns the VALUE for the metric kept at `node'
 */
int metric_node_value(struct metric_node_t *node);
/*
 * Returns the MIN value for the metric kept at `node'
 */
int metric_node_min(struct metric_node_t *node);
/*
 * Returns the MAX value for the metric kept at `node'
 */
int metric_node_max(struct metric_node_t *node);
#endif
